import logging

import ma.api.base_class

_LOGGER = logging.getLogger()


class MagentoInfoApi(ma.api.base_class.Api):
    def get_info(self):
        l = self.magento.core_magento.info()
        return l



import logging
import collections

import ma.api.base_class
import ma.entities.product
import ma.utility

_LOGGER = logging.getLogger()


class CatalogProductApi(ma.api.base_class.Api):
    def get_info_with_id(self, product_id):
        i = self.magento.catalog_product.info(product_id)

        return i

    def get_info_with_sku(self, product_sku):

# NOTE(dustin): This is currently the same request as _id(), but we want to
#               keep the calls separate just in case Magento finally gets their 
#               act together.
        i = self.magento.catalog_product.info(product_sku)

        return i

    def get_list(self, categories=[]):
        filters = {}

# TODO(dustin): We can't get this to work right and Magento isn't cooperating 
#               with our debugging effort.
        assert not categories, "A category filter is not currently supported."

#        if categories:
#            filters['category_id'] = { 'eq': '10' }#in': ','.join([str(c) for c in categories]) }

        l = self.magento.catalog_product.list(filters)
        return l

    def create_product(self, product_type, attribute_set_id, sku, 
                       catalog_product_create_entity):
        cpce_dict = \
            ma.utility.get_dict_from_named_tuple(
                catalog_product_create_entity)

        product_id = \
            self.magento.catalog_product.create(
                product_type, 
                attribute_set_id, 
                sku, 
                cpce_dict)

        product_id = int(product_id)

        _LOGGER.info("Created product with ID (%d).", product_id)

        return product_id



import logging

import ma.api.base_class

_LOGGER = logging.getLogger()


class CatalogProductTypeApi(ma.api.base_class.Api):
    def get_list(self):
        l = self.magento.catalog_product_type.list()
        return l



import json

def pretty_print(results):
    print(json.dumps(
            results, 
            sort_keys=True,
            indent=4, 
            separators=(',', ': ')))

def get_dict_from_named_tuple(nt):
    a = dir(nt)
    d = { 
            k: getattr(nt, k) 
            for k 
            in a 
            if k[0] != '_' \
                and k != 'count' \
                and k != 'index' 
        }

    return d



import logging

import ma.api.base_class

_LOGGER = logging.getLogger()


class CatalogInventoryStockItemApi(ma.api.base_class.Api):
    def get_list_with_ids(self, product_ids, do_allow_missing=False):
        assert product_ids, "Please a nonempty list of product-IDs."

        l = self.magento.cataloginventory_stock_item.list(product_ids)

        assert \
            do_allow_missing is True or len(l) == len(product_ids), \
            "One or more inventory products doesn't exist."

        return l

    def get_list_with_skus(self, product_skus, do_allow_missing=False):
        assert product_skus, "Please a nonempty list of product-SKUs."

# NOTE(dustin): This is currently the same request as _id(), but we want to
#               keep the calls separate just in case Magento finally gets their 
#               act together.
        l = self.magento.cataloginventory_stock_item.list(product_skus)

        assert \
            do_allow_missing is True or len(l) == len(product_skus), \
            "One or more inventory products doesn't exist."

        return l



import os
import logging

_LOGGER = logging.getLogger(__name__)

SCHEME = os.environ.get('MAGENTO_SCHEME', 'https')
USERNAME = os.environ['MAGENTO_USERNAME']
PASSWORD = os.environ['MAGENTO_PASSWORD']
HOSTNAME = os.environ['MAGENTO_HOSTNAME']
PORT = int(os.environ.get('MAGENTO_PORT', '443'))

_LOGGER.debug("CONFIG: S=[%s] U=[%s] P=[%s] H=[%s] P=(%d)",
              SCHEME, USERNAME, PASSWORD, HOSTNAME, PORT)



import logging

_LOG_FORMAT = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'

def configure(is_debugging):
    logger = logging.getLogger()

    if is_debugging is True:
        logger.setLevel(logging.DEBUG)
    else:
        logger.setLevel(logging.INFO)

    sh = logging.StreamHandler()
    formatter = logging.Formatter(_LOG_FORMAT)
    sh.setFormatter(formatter)
    logger.addHandler(sh)



import logging

import ma.api.base_class

_LOGGER = logging.getLogger()


class CatalogCategoryApi(ma.api.base_class.Api):
    def get_info(self, category_id):
        i = self.magento.catalog_category.info(category_id)

        return i

    def assigned_products(self, category_id):
        i = self.magento.catalog_category.assignedProducts(category_id)
        return i

    def assign_product_with_id(self, category_id, product_id):
        r = self.magento.catalog_category.assignProduct(category_id, product_id)
        assert r == True, "Could not assign (by ID)"

    def assign_product_with_sku(self, category_id, product_sku):

# NOTE(dustin): This is currently the same request as _id(), but we want to
#               keep the calls separate just in case Magento finally gets their 
#               act together.
        r = self.magento.catalog_category.assignProduct(category_id, product_sku)
        assert r == True, "Could not assign (by SKU)"

    def get_tree(self, parent_id=1):
        t = self.magento.catalog_category.tree()
        return t



import collections

CATALOG_PRODUCT_CREATE_ENTITY = \
    collections.namedtuple(
        'CATALOG_PRODUCT_CREATE_ENTITY', [
            'categories',
            'websites',
            'name',
            'description',
            'short_description',
            'weight',
            'status',
            'url_key',
            'url_path',
            'visibility',
            'category_ids',
            'website_ids',
            'has_options',
            'gift_message_available',
            'price',
            'special_price',
            'special_from_date',
            'special_to_date',
            'tax_class_id',
            'tier_price',
            'meta_title',
            'meta_keyword',
            'meta_description',
            'custom_design',
            'custom_layout_update',
            'options_container',
            'additional_attributes',
            'stock_data',
        ])

def build_catalog_product_create_entity(*args, **kwargs):

    t = CATALOG_PRODUCT_CREATE_ENTITY(*args, **kwargs)

    # Add any validation here.

    return t

CATALOG_PRODUCT_TIER_PRICE_ENTITY = \
    collections.namedtuple(
        'CATALOG_PRODUCT_TIER_PRICE_ENTITY', [
            'customer_group_id',
            'website',
            'qty',
            'price',
        ])

def catalog_product_tier_price_entity(*args, **kwargs):

    t = CATALOG_PRODUCT_TIER_PRICE_ENTITY(*args, **kwargs)

    # Add any validation here.

    return t

CATALOG_PRODUCT_ADDITIONAL_ATTRIBUTES_ENTITY = \
    collections.namedtuple(
        'CATALOG_PRODUCT_TIER_PRICE_ENTITY', [
            'multi_data',
            'single_data',
        ])

def catalog_product_additional_attributes_entity(*args, **kwargs):

    t = CATALOG_PRODUCT_ADDITIONAL_ATTRIBUTES_ENTITY(*args, **kwargs)

    # Add any validation here.

    return t

CATALOG_INVENTORY_STOCK_ITEM_UPDATE_ENTITY = \
    collections.namedtuple(
        'CATALOG_INVENTORY_STOCK_ITEM_UPDATE_ENTITY', [
            'qty',
            'is_in_stock',
            'manage_stock',
            'use_config_manage_stock',
            'min_qty',
            'use_config_min_qty',
            'min_sale_qty',
            'use_config_min_sale_qty',
            'max_sale_qty',
            'use_config_max_sale_qty',
            'is_qty_decimal',
            'backorders',
            'use_config_backorders',
            'notify_stock_qty',
            'use_config_notify_stock_qty',
        ])

def catalog_inventory_stock_item_update_entity(*args, **kwargs):

    t = CATALOG_INVENTORY_STOCK_ITEM_UPDATE_ENTITY(*args, **kwargs)

    # Add any validation here.

    return t



import logging

import ma.api.base_class

_LOGGER = logging.getLogger()


class CatalogProductTagApi(ma.api.base_class.Api):
    def get_list(self, product_id):
        l = self.magento.catalog_product_tag.list(product_id)
        return l



import logging

import ma.api.base_class

_LOGGER = logging.getLogger()


class CatalogProductAttributeApi(ma.api.base_class.Api):
    def get_list(self, attribute_set_id):
        l = self.magento.catalog_product_attribute.list(attribute_set_id)
        return l



import logging

import ma.constants
import ma.api.base_class

_LOGGER = logging.getLogger()


class SalesOrderInvoiceApi(ma.api.base_class.Api):
    def get_transactions(self, from_timestamp, to_timestamp):
        format = ma.constants.TIMESTAMP_FORMAT_1

        from_timestamp_phrase = from_timestamp.strftime(format)
        to_timestamp_phrase = to_timestamp.strftime(format)

        _LOGGER.debug("Returning transactions from [%s] to [%s].", 
                      from_timestamp_phrase, to_timestamp_phrase)

        filters = {
            'created_at': {
                'from': from_timestamp_phrase,
                'to': to_timestamp_phrase,
            }, 
        }

        l = self.magento.sales_order_invoice.list(filters)
        return l



import logging

import ma.api.base_class

_LOGGER = logging.getLogger()


class CatalogProductAttributeSetApi(ma.api.base_class.Api):
    def get_list(self):
        l = self.magento.catalog_product_attribute_set.list()
        for record in l:
            record['set_id'] = int(record['set_id'])

            yield record



import logging

import ma.api.base_class

_LOGGER = logging.getLogger()


class CatalogProductAttributeMediaApi(ma.api.base_class.Api):
    def get_list_with_product_id(self, product_id):
        l = self.magento.catalog_product_attribute_media.list(product_id)

        # Example return:
        #
        #[
        #    {
        #        "exclude": "0",
        #        "file": "/m/s/msj003t_1.jpg",
        #        "label": "",
        #        "position": "1",
        #        "types": [
        #            "image",
        #            "small_image",
        #            "thumbnail"
        #        ],
        #        "url": "http://dev.bugatchi.com/media/catalog/product/m/s/msj003t_1.jpg"
        #    },
        #    {
        #        "exclude": "0",
        #        "file": "/m/s/msj003a_1.jpg",
        #        "label": "",
        #        "position": "2",
        #        "types": [],
        #        "url": "http://dev.bugatchi.com/media/catalog/product/m/s/msj003a_1.jpg"
        #    },
        #]

        return l

    def get_list_with_product_sku(self, product_sku):
# NOTE(dustin): This is currently the same request as _id(), but we want to
#               keep the calls separate just in case Magento finally gets their 
#               act together.
        l = self.magento.catalog_product_attribute_media.list(product_sku)
        return l



import logging

import ma.api.base_class

_LOGGER = logging.getLogger()


class CustomerGroupApi(ma.api.base_class.Api):
    def get_list(self):
        l = self.magento.customer_group.list()

        for record in l:
            record['customer_group_id'] = int(record['customer_group_id'])
            yield record



import os
import logging

import magento

import ma.config.magento

_TIMESTAMP_FORMAT = '%Y-%m-%d %H:%M:%S'

_LOGGER = logging.getLogger(__name__)


class _ApiResources(object):
    def __init__(self):
        self.__ar = self.__get_api_resource()

    def __get_server_config(self):
        scheme = ma.config.magento.SCHEME
        username = ma.config.magento.USERNAME
        password = ma.config.magento.PASSWORD
        hostname = ma.config.magento.HOSTNAME
        port = ma.config.magento.PORT

        return {
            'scheme': scheme,
            'username': username,
            'password': password,
            'hostname': hostname,
            'port': port,
        }

    def __get_api_resource(self):
        c = self.__get_server_config()

        m = magento.MagentoAPI(
                c['hostname'], 
                c['port'], 
                c['username'], 
                c['password'],
                proto=c['scheme'])

        return m

    @property
    def api_resource(self):
        return self.__ar


_API_RESOURCES = None
def _get_api_resources():
    global _API_RESOURCES
    if _API_RESOURCES is None:
        _API_RESOURCES = _ApiResources()

    return _API_RESOURCES


class Api(object):
    def __init__(self):
        ars = _get_api_resources()
        self.__ar = ars.api_resource

    @property
    def magento(self):
        return self.__ar
